/* Calendar Macro
   Creates calendars on FinalWriter4 (SoftWood)
   Written by Ronald Goertz
   based on a program by Andrew McCardie
   $VER: Calendar v1.1 (11 Mar 96)
*/
 
OPTIONS RESULTS

/* User-defined variables */
Extended         = 1
MiniCal          = 1

ShadowColor      = "Cyan"
MonthFont        = "FWFonts/SWOLFonts/Serif/Dragon"
MonthFSize       = 44
MonthFWidth      = 150
WeekdayFont      = "FWFonts/SWOLFonts/Serif/Dragon"
WeekdayFSize     = 22
WeekdayFWidth    = 80
DateFont         = "SoftSans"
DateFSize        = 10
DateFWidth       = 100
HighlightFSize   = 8
MiniCalFSize     = 8

/* Program variables */
ClearDoc
GetPageSetup Orient
if result == "Tall" then do
   PageSetup ORIENT Wide
end
GetPageSetup Width Height Top Bottom Left Right
parse var result FullWidth FullHeight TopMargin BottomMargin LeftMargin RightMargin

GetDisplayPrefs Measure; Units = result
select
   when Units = 'Inches' then PointsPerUnit = 72
   when Units = 'Metric' then PointsPerUnit = 28.34645669291
   when Units = 'Pica' then   PointsPerUnit = 12
end

MonthFontHeight     = MonthFSize / PointsPerUnit
WeekdayFontHeight   = WeekdayFSize / PointsPerUnit
DateFontHeight      = DateFSize / PointsPerUnit
HighlightFontHeight = HighlightFSize / PointsPerUnit
MiniCalFontHeight   = MiniCalFSize / PointsPerUnit

PageWidth        = FullWidth - LeftMargin - RightMargin
PageHeight       = FullHeight - TopMargin - BottomMargin
TextArea         = .15 * PageHeight
BoxWidth         = PageWidth/7
BoxHeight        = (PageHeight-TextArea)/5
 
DateDown         = DateFontHeight
DateOffset       = 3.6 / PointsPerUnit
HighlightOffset  = 14.4 / PointsPerUnit
MiniCalWidth     = 108 / PointsPerUnit

Day.0            = 'Sunday'
Day.1            = 'Monday'
Day.2            = 'Tuesday'
Day.3            = 'Wednesday'
Day.4            = 'Thursday'
Day.5            = 'Friday'
Day.6            = 'Saturday'

Month.1          = 'January'
Month.2          = 'February'
Month.3          = 'March'
Month.4          = 'April'
Month.5          = 'May'
Month.6          = 'June'
Month.7          = 'July'
Month.8          = 'August'
Month.9          = 'September'
Month.10         = 'October'
Month.11         = 'November'
Month.12         = 'December'
 
MonthLength.1    = 31
MonthLength.2    = 28
MonthLength.3    = 31
MonthLength.4    = 30
MonthLength.5    = 31
MonthLength.6    = 30
MonthLength.7    = 31
MonthLength.8    = 31
MonthLength.9    = 30
MonthLength.10   = 31
MonthLength.11   = 30
MonthLength.12   = 31
 
Do i = 1 to 12
   If Date('M') == Month.i Then ThisMonth = i
End

RequestText '"Month Select" "Month:" 'DATE('M')''
MonthName = RESULT
if MonthName = "RESULT" | MonthName = "" then Exit 10

Month = -1
if Datatype(MonthName) == 'NUM' then do
   if MonthName > 0 & MonthName < 13 then Month = MonthName
end
else
   Do i = 1 to 12
      If MonthName == Month.i Then Month = i
   End
If Month == -1 then Exit 10

Drop RESULT
Year = SUBSTR(DATE('S'),1,4)
if Month < ThisMonth then Year = Year + 1
RequestText '"Year Select" "Year:" 'Year''
Year = RESULT
if Year = "RESULT" | Year = "" then Exit 10

View 400

TextBlockPrefs TEXTFLOW None
call AssignHighlights

/* Finally, the program */
   If Month < 10 then TempDate = Year || "0" || Month || "01"
   else TempDate = Year || Month || "01"
 
   if (Year//4 == 0 & Year//100 > 0) | Year//400 == 0 Then MonthLength.2 = 29
 
   StartDate = Date('W', TempDate, 'S')
 
   Select
      When StartDate == "Sunday"    Then StartDate = 0
      When StartDate == "Monday"    Then StartDate = 1
      When StartDate == "Tuesday"   Then StartDate = 2
      When StartDate == "Wednesday" Then StartDate = 3
      When StartDate == "Thursday"  Then StartDate = 4
      When StartDate == "Friday"    Then StartDate = 5
      When StartDate == "Saturday"  Then StartDate = 6
   End
 
   TextBlockTypePrefs COLOR White
   TextBlockTypePrefs SIZE 4
   DrawTextBlock 1 1 1 TempDate /* Object ID == 2, only required for AddEvent macro, MUST be drawn first */
 
   /* Required whether or not mini calendars are used */
   PrevMonth = Month - 1
   PrevYear  = Year
   if PrevMonth == 0 then do
      PrevMonth = 12
      PrevYear  = Year - 1
   end
 
   if MiniCal then do
      /* Required for both mini calendars */
      TextBlockTypePrefs SIZE MiniCalFSize COLOR Black
      BoxPrefs LINEWT Hairline LINECOLOR Black FILL Transparent
      DrawTextBlock 1 1 1 "1"; ID = RESULT
      Redraw
      GetObjectCoords ID; parse var RESULT . . . WidthOf1 .
      DeleteObject ID
 
      /* Draw mini calendar for previous month */
      MiniCalLeft = LeftMargin + MiniCalWidth/28
      DrawTextBlock 1 1 (TopMargin + MiniCalFontHeight) Month.PrevMonth || " " || PrevYear; ID.0 = RESULT
      Redraw
      GetObjectCoords ID.0; Parse var RESULT . . Top Width Height
      SetObjectCoords ID.0 1 (MiniCalLeft - MiniCalWidth/28+ (MiniCalWidth - Width)/2) Top Width Height
      StartColumn = StartDate - MonthLength.PrevMonth//7
      If StartColumn < 0 then StartColumn = StartColumn + 7
      Column = StartColumn
      Day = 0
      Row = 2
      Do Until Day = MonthLength.PrevMonth
         Day = Day + 1
         If Day < 10 then Do
            DrawTextBlock 1 (MiniCalLeft+Column*MiniCalWidth/7 + WidthOf1) (TopMargin + Row*MiniCalFontHeight) Day; ID.Day = RESULT
         End
         else Do
            DrawTextBlock 1 (MiniCalLeft+Column*MiniCalWidth/7) (TopMargin + Row*MiniCalFontHeight) Day; ID.Day = RESULT
         End
         Column = Column + 1
         if Column == 7 then Do
            Column = 0
            Row = Row + 1
         End
      End
      Rows = trunc((StartColumn + MonthLength.PrevMonth)/7) + 1.5
      if (StartColumn + MonthLength.PrevMonth)//7 > 0 then Rows = Rows + 1
      DrawBox 1 (MiniCalLeft - MiniCalWidth/28) TopMargin MiniCalWidth (Rows*MiniCalFontHeight)
      Redraw
      do i = 0 to MonthLength.PrevMonth
         SelectObject ID.i MULTIPLE
      End
      Group
 
      /* Draw mini calendar for next month */
      MiniCalLeft = FullWidth - RightMargin - MiniCalWidth
      NextMonth = Month + 1
      NextYear  = Year
      if NextMonth == 13 then Do
         NextMonth = 1
         NextYear  = Year + 1
      End
      DrawTextBlock 1 1 (TopMargin + MiniCalFontHeight) Month.NextMonth || " " || NextYear; ID.0 = RESULT
      Redraw
      GetObjectCoords ID.0; Parse var RESULT . . Top Width Height
      SetObjectCoords ID.0 1 (MiniCalLeft - MiniCalWidth/28+ (MiniCalWidth - Width)/2) Top Width Height
      StartColumn = StartDate + MonthLength.Month//7
      If StartColumn > 6 then StartColumn = StartColumn - 7
      Column = StartColumn
      Day = 0
      Row = 2
      Do Until Day = MonthLength.NextMonth
         Day = Day + 1
         If Day < 10 then Do
            DrawTextBlock 1 (MiniCalLeft+Column*MiniCalWidth/7 + WidthOf1) (TopMargin + Row*MiniCalFontHeight) Day; ID.Day = RESULT
         End
         else Do
            DrawTextBlock 1 (MiniCalLeft+Column*MiniCalWidth/7) (TopMargin + Row*MiniCalFontHeight) Day; ID.Day = RESULT
         End
         Column = Column + 1
         if Column == 7 then Do
            Column = 0
            Row = Row + 1
         End
      End
      Rows = trunc((StartColumn + MonthLength.NextMonth)/7) + 1.5
      if (StartColumn + MonthLength.NextMonth)//7 > 0 then Rows = Rows + 1
      DrawBox 1 (MiniCalLeft - MiniCalWidth/28) TopMargin MiniCalWidth (Rows*MiniCalFontHeight)
      Redraw
      do i = 0 to MonthLength.NextMonth
         SelectObject ID.i MULTIPLE
      End
      Group
   End /* MiniCal section */

   /* Draw dates and optional highlights */
   i = 0
   Day = 0
   PrevDay = MonthLength.PrevMonth - StartDate
   NextDay = 0
   TextBlockTypePrefs WIDTH DateFWidth FONT DateFont
   Do while (i < 5)
      j = 0
      BoxTop  = TopMargin + TextArea + BoxHeight*i
                 
      Do while (j < 7)
         j_plus_1 = j + 1
         BoxLeft = LeftMargin + BoxWidth * j
         SerialPosition = (i * 7) + j
         if (SerialPosition >= StartDate) & (SerialPosition < StartDate + MonthLength.Month) then Do
            Day = Day + 1
            TextBlockTypePrefs SIZE DateFSize COLOR Black
            DrawTextBlock 1 (BoxLeft + DateOffset) (BoxTop + DateDown) Day
            if Highlight.Month.Day ~= upper("Highlight." || Month || "." || Day) Then Do
               TextBlockTypePrefs SIZE HighlightFSize
               TextLeft   = (BoxLeft + DateOffset) + HighlightOffset
               TextBottom = (BoxTop + DateDown - (DateFontHeight - HighlightFontHeight)/2)
               DrawTextBlock 1 TextLeft TextBottom Highlight.Month.Day
            End
            Select
               when i == 0 then LineTop.j = TopMargin + TextArea
               when i == 4 then LineBottom.j_plus_1 = TopMargin + TextArea + BoxHeight*5
               otherwise nop
            End
         End
         else Do
            if Extended then Do
               if SerialPosition < StartDate then Do
                  PrevDay = PrevDay + 1
                  PrintDay = PrevDay
               End
               else Do
                  NextDay = NextDay + 1
                  PrintDay = NextDay
               End
               TextBlockTypePrefs COLOR ShadowColor
               TextBlockTypePrefs SIZE DateFSize
               DrawTextBlock 1 (BoxLeft + DateOffset) (BoxTop + DateDown) PrintDay
            End
            Select
               when i == 0 then LineTop.j = TopMargin + TextArea + BoxHeight
               when i == 4 then LineBottom.j_plus_1 = TopMargin + TextArea + BoxHeight*4
            End
         End
         j = j + 1
      End
      i = i + 1
   End
 
   /* Special case for months whose last day(s) wrap to the beginning of the week */
   BoxesDrawn = 35
   RequiredBoxes = StartDate + MonthLength.Month
   Do while RequiredBoxes > BoxesDrawn
      BoxLeft   = LeftMargin + (BoxesDrawn - 35) * BoxWidth
      BoxRight  = BoxLeft + BoxWidth
      BoxTop    = TopMargin + TextArea + (BoxHeight * 4.5)
      BoxBottom = BoxTop
      DrawLine 1 BoxLeft BoxTop BoxRight BoxBottom
      Day = Day + 1
      DrawTextBlock 1 (BoxLeft + DateOffset) (BoxTop + DateDown) Day
      if Highlight.Month.Day ~= upper("Highlight." || Month || "." || Day) Then Do
         TextBlockTypePrefs SIZE HighlightFSize
         TextLeft   = (BoxLeft + DateOffset) + HighlightOffset
         TextBottom = (BoxTop + DateDown - (DateFSize - HighlightFSize)*5)
         DrawTextBlock 1 TextLeft TextBottom Highlight.Month.Day
      End
      BoxesDrawn = BoxesDrawn + 1
   End
 
   /* Draw vertical grid */
   if StartDate + MonthLength.Month > 28 then LineBottom.0 = TopMargin + TextArea + BoxHeight*5
   else LineBottom.0 = TopMargin + TextArea + BoxHeight*4
   LineTop.7 = TopMargin + TextArea
   LastLongLine = 7
   do i = 0 to 7
      x1 = LeftMargin + BoxWidth*i
      LinePrefs LINECOLOR ShadowColor
      if (LineTop.i > TopMargin + TextArea) & Extended then
         DrawLine 1 x1 (TopMargin + TextArea) x1 LineTop.i
      if (LineBottom.i = TopMargin + TextArea + BoxHeight*4) & Extended then
         DrawLine 1 x1 (TopMargin + TextArea + BoxHeight*4) x1 (TopMargin + TextArea + BoxHeight*5)
      LinePrefs LINECOLOR Black
      DrawLine 1 x1 LineTop.i x1 LineBottom.i
      if (LastLongLine == 7) & (LineBottom.i == TopMargin + TextArea + BoxHeight*4) then
         LastLongLine = i - 1
   end
 
   /* Draw horizontal grid */
   y1 = TopMargin + TextArea
   LinePrefs LINECOLOR ShadowColor
   If Extended then DrawLine 1 LeftMargin y1 (LeftMargin + BoxWidth*StartDate) y1
   LinePrefs LINECOLOR Black
   DrawLine 1 (LeftMargin + BoxWidth*StartDate) y1 (LeftMargin + BoxWidth*7) y1
   do i = 1 to 4
      y1 = TopMargin + TextArea + BoxHeight*i
      DrawLine 1 LeftMargin y1 (LeftMargin + BoxWidth*7) y1
   end
   y1 = TopMargin + TextArea + BoxHeight*5
   if StartDate + MonthLength.Month > 28 then Do
      LinePrefs LINECOLOR ShadowColor
      If Extended then DrawLine 1 (LeftMargin + BoxWidth*LastLongLine) y1 (LeftMargin + BoxWidth*7) y1
      LinePrefs LINECOLOR Black
      DrawLine 1 LeftMargin y1 (LeftMargin + BoxWidth*LastLongLine) y1
   end
   else if Extended then Do
      LinePrefs LINECOLOR ShadowColor
      DrawLine LeftMargin y1 RightMargin y1
   end
 
   /* Create month/year header  */
   TextBlockTypePrefs COLOR Black
   TextBlockTypePrefs SIZE MonthFSize
   TextBlockTypePrefs WIDTH MonthFWidth
   TextBlockTypePrefs FONT MonthFont
   DrawTextBlock 1 LeftMargin (TopMargin + MonthFontHeight) upper(MonthName || " " || Year); Month_obj = RESULT

   /* Create weekday titles */
   TextBlockTypePrefs SIZE WeekdayFSize
   TextBlockTypePrefs WIDTH WeekdayFWidth
   TextBlockTypePrefs FONT WeekdayFont
   i = 0
   Do while (i < 7)
      DrawTextBlock 1 1 1 upper(Day.i); text_obj.i = RESULT
      i = i + 1
   End

   Redraw
 
   /* Position month/year header */
   GetObjectCoords Month_obj; Parse Var result . . TextBottom TextWidth TextHeight
   TextLeft = (LeftMargin + (PageWidth - TextWidth)/2)
   SetObjectCoords Month_obj 1 TextLeft TextBottom TextWidth TextHeight
    
   /* Position weekday titles */
   TextBottom = TopMargin + TextArea - WeekdayFontHeight * .6
   i = 0
   Do while (i < 7)
      GetObjectCoords text_obj.i; Parse Var result . . . TextWidth TextHeight
      BoxCenter = (LeftMargin + (i * BoxWidth) + BoxWidth/2)
      TextLeft = (BoxCenter - TextWidth/2)
      SetObjectCoords text_obj.i 1 TextLeft TextBottom TextWidth TextHeight
      i = i + 1
   End
 
 SelectObject
 View 75
exit

AssignHighlights:
/* January Highlights */
   Highlight.1.1    = "New Year's Day"
   /* Find Martin Luther King Day: 3rd Monday in January */
      first = Date('W', Year || "0101", 'S')
      Select
         When first == "Sunday"    Then Day = 16
         When first == "Monday"    Then Day = 17
         When first == "Tuesday"   Then Day = 21
         When first == "Wednesday" Then Day = 20
         When first == "Thursday"  Then Day = 19
         When first == "Friday"    Then Day = 18
         When first == "Saturday"  Then Day = 17
      End
      Highlight.1.Day = "MLK Day"

/* February Highlights */
   Highlight.2.2  = "Groundhog Day"
   Highlight.2.14 = "Valentines' Day"
   /* Find Presidents' Day: 3rd Monday in February */
      first = Date('W', Year || "0201", 'S')
      Select
         When first == "Sunday"    Then Day = 16
         When first == "Monday"    Then Day = 15
         When first == "Tuesday"   Then Day = 21
         When first == "Wednesday" Then Day = 20
         When first == "Thursday"  Then Day = 19
         When first == "Friday"    Then Day = 18
         When first == "Saturday"  Then Day = 17
      End
      Highlight.2.Day = "Presidents' Day"

/* March Highlights */
   Highlight.3.17 = "St. Patrick's Day"

/* April Highlights */
   /* Find Start DST: First Sunday in April */
      first = Date('W', Year || "0401", 'S')
      Select
         When first == "Sunday"    Then Day = 1
         When first == "Monday"    Then Day = 7
         When first == "Tuesday"   Then Day = 6
         When first == "Wednesday" Then Day = 5
         When first == "Thursday"  Then Day = 4
         When first == "Friday"    Then Day = 3
         When first == "Saturday"  Then Day = 2
      End
      Highlight.4.Day = "Start DST"
   /* Find Tax Day: Apr 15 or first workday after */
      first = Date('W', Year || "0401", 'S')
      Select
         When first == "Sunday"    Then Day = 16
         When first == "Monday"    Then Day = 15
         When first == "Tuesday"   Then Day = 15
         When first == "Wednesday" Then Day = 15
         When first == "Thursday"  Then Day = 15
         When first == "Friday"    Then Day = 15
         When first == "Saturday"  Then Day = 17
      End
      Highlight.4.Day = "Tax Day"
   /* Find Arbor Day: Last Friday in April */
      first = Date('W', Year || "0401", 'S')
      Select
         When first == "Sunday"    Then Day = 27
         When first == "Monday"    Then Day = 26
         When first == "Tuesday"   Then Day = 25
         When first == "Wednesday" Then Day = 24
         When first == "Thursday"  Then Day = 30
         When first == "Friday"    Then Day = 29
         When first == "Saturday"  Then Day = 28
      End
      Highlight.4.Day = "Arbor Day"

/* May Highlights */
   /* Find Mother's Day: 2nd Sunday in May */
      first = Date('W', Year || "0501", 'S')
      Select
         When first == "Sunday"    Then Day = 8
         When first == "Monday"    Then Day = 14
         When first == "Tuesday"   Then Day = 13
         When first == "Wednesday" Then Day = 12
         When first == "Thursday"  Then Day = 11
         When first == "Friday"    Then Day = 10
         When first == "Saturday"  Then Day = 9
      End
      Highlight.5.Day = "Mother's Day"

   /* Find Memorial Day: Last Monday in May */
      first = Date('W', Year || "0501", 'S')
      Select
         When first == "Sunday"    Then Day = 30
         When first == "Monday"    Then Day = 29
         When first == "Tuesday"   Then Day = 28
         When first == "Wednesday" Then Day = 27
         When first == "Thursday"  Then Day = 26
         When first == "Friday"    Then Day = 25
         When first == "Saturday"  Then Day = 31
      End
      Highlight.5.Day = "Memorial Day"

/* June Highlights */
   Highlight.6.14 = "Flag Day"
   /* Find Father's Day: 3rd Sunday in June */
      first = Date('W', Year || "0601", 'S')
      Select
         When first == "Sunday"    Then Day = 22
         When first == "Monday"    Then Day = 21
         When first == "Tuesday"   Then Day = 20
         When first == "Wednesday" Then Day = 19
         When first == "Thursday"  Then Day = 18
         When first == "Friday"    Then Day = 17
         When first == "Saturday"  Then Day = 16
      End
      Highlight.6.Day = "Father's Day"

/* July Highlights */
   Highlight.7.4  = "Independence Day"

/* August Highlights */

/* September Highlights */
   /* Find Labor Day: First Monday in September */
      first = Date('W', Year || "0901", 'S')
      Select
         When first == "Sunday"    Then Day = 2
         When first == "Monday"    Then Day = 8
         When first == "Tuesday"   Then Day = 7
         When first == "Wednesday" Then Day = 6
         When first == "Thursday"  Then Day = 5
         When first == "Friday"    Then Day = 4
         When first == "Saturday"  Then Day = 3
      End
      Highlight.9.Day = "Labor Day"

   /* Find Grandparent's Day: First Sunday after Labor Day */
      first = Date('W', Year || "0901", 'S')
      Select
         When first == "Sunday"    Then Day = 8
         When first == "Monday"    Then Day = 14
         When first == "Tuesday"   Then Day = 13
         When first == "Wednesday" Then Day = 12
         When first == "Thursday"  Then Day = 11
         When first == "Friday"    Then Day = 10
         When first == "Saturday"  Then Day = 9
      End
      Highlight.9.Day = "Grandparent's Day"

/* October Highlights */
   Highlight.10.15 = "Sweetest Day"
   Highlight.10.31 = "Halloween"
   /* Find Columbus Day: 2nd Monday in October */
      first = Date('W', Year || "1001", 'S')
      Select
         When first == "Sunday"    Then Day = 9
         When first == "Monday"    Then Day = 8
         When first == "Tuesday"   Then Day = 14
         When first == "Wednesday" Then Day = 13
         When first == "Thursday"  Then Day = 12
         When first == "Friday"    Then Day = 11
         When first == "Saturday"  Then Day = 10
      End
      Highlight.10.Day = "Columbus Day"
   /* Find Mother-in-Law's Day: 4th Sunday in October */
      first = Date('W', Year || "1001", 'S')
      Select
         When first == "Sunday"    Then Day = 22
         When first == "Monday"    Then Day = 28
         When first == "Tuesday"   Then Day = 27
         When first == "Wednesday" Then Day = 26
         When first == "Thursday"  Then Day = 25
         When first == "Friday"    Then Day = 24
         When first == "Saturday"  Then Day = 23
      End
      Highlight.10.Day = "Mother-in-Law's Day"
   /* Find End DST: Last Sunday in October */
      first = Date('W', Year || "1001", 'S')
      Select
         When first == "Sunday"    Then Day = 29
         When first == "Monday"    Then Day = 28
         When first == "Tuesday"   Then Day = 27
         When first == "Wednesday" Then Day = 26
         When first == "Thursday"  Then Day = 25
         When first == "Friday"    Then Day = 31
         When first == "Saturday"  Then Day = 30
      End
      Highlight.10.Day = "End DST"

/* November Highlights */
   Highlight.11.11 = "Veteran's Day"
   /* Find Thanksgiving: 4th Thursday in November */
      first = Date('W', Year || "1101", 'S')
      Select
         When first == "Sunday"    Then Day = 26
         When first == "Monday"    Then Day = 25
         When first == "Tuesday"   Then Day = 24
         When first == "Wednesday" Then Day = 23
         When first == "Thursday"  Then Day = 22
         When first == "Friday"    Then Day = 28
         When first == "Saturday"  Then Day = 27
      End
      Highlight.11.Day = "Thanksgiving"

/* December Highlights */
   Highlight.12.25 = "Christmas"

return
